;
;**************************************************************************************************
;DOS/65 C02 (CMOS) RAM Version
;
;This is Version 3.20 Beta (KM 31/01/2023)
;This is the bootable image for the RAM loaded version of DOS/65 (CMOS version)
;
; Based on Richard A. Leary's DOS/65 V3 code - many thanks Richard!!
;
; The target platform for this build is:
;  A prototype 3.3-Volt system based on the C02 Pocket SBC - KM
;  This prototype uses a NXP 28L92 DUART and a Hitachi MicroDrive PATA interface
;  The system has been modified for more RAM, now 56KB RAM and 8KB ROM (less I/O), 8MHz CPU
;
;**************************************************************************************************
; Assembler directives
;
                PW      132
                PL      66
                CHIP    W65C02S ;Enable WDC 65C02 instructions
                GLOBALS ON
                CASE    OFF
                INCLIST ON
;
;**************************************************************************************************
;
; This version uses many CMOS instructions and addressing modes!
;
; There are multiple modules that make up this bootable version and use "include" to
; add them into the main code. The Code Modules are:
;       DOS65-MAIN-V320.ASM (This file)
;       DOS65-CCM-V320.ASM - Concurrent Command Module
;       DOS65-PEM-V320.ASM - Primitive Execution Module
;       DOS65-SIM-V320.ASM - System Interface Module
;
; Other files required contain system variables and constants for C02BIOS and C02Monitor,
; which are included in the SIM source module.
;       C02Constants2.asm - C02 BIOS/Monitor constants file
;       C02JMP_Table.asm - C02 BIOS/Monitor JMP table for routines
;
;  The current BIOS and Monitor versions for the Prototype system are 4.02
;**************************************************************************************************
;
;               31 January 2023 - KM
;Major revision! This is an entirely RAM based version of DOS/65
;
; Based on my 3.04 CMOS ROM Version
; - All data areas are included in the linked modules
; - Allocation Maps for 8 drives are zeroed and adds 4KB to the 6KB of DOS/65 code
; - Supports 8- 8MB drives configured on the Microdrive.
;
; ALL modules (CCM, PEM, SIM) have been reworked for RAM base and include data areas.
; - Page Zero usage has been reduced - currently 10 bytes are used.
;
; Major update to this version:
; - Traditional Track/Sector call structure has been eliminated!
; - All calls for disk drives are at a record level.
; - PEM has been changed to only make calls for record numbers
; - SIM has been re-written to accept record numbers and perform blocking/de-blocking for LBAs
;**************************************************************************************************
;
;Base addresses and definitions
pzstrt          equ     $90             ;start of page zero system RAM
btejmp          equ     $100            ;warm boot jump
pemjmp          equ     $103            ;jump to PEM
iostat          equ     $106            ;I/O status
dflfcb          equ     $107            ;default FCB
dflbuf          equ     $128            ;default buffer
tea             equ     $800            ;TEA start
;
;C02 Pocket SBC memory parameters - 3.3V prototype
; NOTE: The ramstrt should be changed to $B800!
; this provides a TEA area from $0800 to $B7FF, which is 44KB
;
ramstrt         equ     $B800           ;RAM location assembly
hstbuf          equ     $0600           ;IDE host buffer
;
;Fixed parameters
lf              equ     $0A             ;linefeeed
cr              equ     $0D             ;return
eof             equ     $1A             ;end of file
null            equ     0               ;null
ctlc            equ     3               ;abort
ctle            equ     5               ;physical cr lf
ctli            equ     9               ;tab character
ctlp            equ     $10             ;toggle printer
ctlr            equ     $12             ;repeat line
ctls            equ     $13             ;freeze
ctlx            equ     $18             ;cancel
semico          equ     $3B             ;semicolon
delete          equ     $7F             ;delete character
numcmd          equ     37              ;number of PEM commands
numvar          equ     8               ;number of page zero variable for switch
;
;Page Zero use
                page0
;CCM / PEM
addinp          equ     pzstrt          ;initialized to a,y
bufadd          equ     addinp+2        ;buffer address
alcpnt          equ     bufadd+2        ;allocation map pointer
chkpnt          equ     alcpnt+2        ;checksum map pointer
;
d65flg          equ     chkpnt+2        ;DOS/65 flag for CCM/PEM usage
                                        ;bit 7 used for blmode flag in PEM
                                        ;bit 6 used for exrwfl flag in PEM
                                        ;bit 5 used for drvflg flag in PEM
                                        ;bit 4 used for outflg flag in PEM
                                        ;bits 3,2,1 unused
                                        ;bit 0 used for submit flag in CCM
;
;SIM
sim_recflg      equ     d65flg+1        ;Disk I/O status flag for SIM
                                        ;bit 7 used for valid LBA
                                        ;bit 6 used for dirty block
                                        ;bit 5 used for read operation
                                        ;bit 4 used for write operation
                                        ;bits 3,2 unused
                                        ;bits 1,0 used for record offset
;
                ends
;
;**************************************************************************************************
;
; Main program code starts here:
        org     ramstrt
;
LOAD_HEADER
;Below is the Load Header required by the BOOT BLOCK!
; it contains details required to complete the loading and
; and configuration of the Boot Image before execution.
; The load header is a 16-byte field located at the start of
; the Boot Image file. It is structured as:
;
; - The 16-bit address to start loading the Boot Image.
; - The 16-bit address to start the Boot Image (BOOT code jumps to this).
; - The 16-bit address for loading the offset to the start of disk data.
; - A long word for the size of the required disk data (for assigned drives).
; - A 5 character string ID to validate the Boot Image.
; - A 1 byte checksum to verify the header itself.
;
;Note that this is not yet being used...
; this will be used when the boot record setup is finalized.
; timing is everything... and the timing for this part is not yet.
;
        .DW     ramstrt         ;Address to load Boot Image
        .DW     SIM             ;Cold Start location for SIM
        .DW     part_offset     ;Location to copy LBA offset
        .LONG   131072          ;64MB for 8-drives (at 8MB each)
;
string_id
        byte    "dos65"         ;ID string to validate Boot image
;
        byte    #$76            ;checksum (must be manually calculated/entered)
;
;**************************************************************************************************
;
        include         DOS65-CCM-V320.ASM      ;Include CCM Module
;
        org     $/256*256+256                   ;Align to Page address
;
        include         DOS65-PEM-V320.ASM      ;Include PEM Module
;
        org     $/256*256+256                   ;Align to Page address
;
        include         DOS65-SIM-V320.ASM      ;Include SIM Module
;
;**************************************************************************************************
;
        org     $/256*256+256                   ;Align to Page address
;
alloc_maps
;allocation and checksum maps
;
;drive a
almpa           equ     alloc_maps
        BLKW    256
;drive b
almpb           equ     almpa+512
        BLKW    256
;drive c
almpc           equ     almpb+512
        BLKW    256
;drive d
almpd           equ     almpc+512
        BLKW    256
;drive e
almpe           equ     almpd+512
        BLKW    256
;drive f
almpf           equ     almpe+512
        BLKW    256
;drive g
almpg           equ     almpf+512
        BLKW    256
;drive h
almph           equ     almpg+512
        BLKW    256
;
        end

